/**
 * Created by IntelliJ IDEA.
 * User: Kilho Shin
 * Date: 11/02/21
 * Time: 11:45
 * To change this template use File | Settings | File Templates.

  This file is part of Mapping Kernels for Trees.

  Mapping Kernels for Trees is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Mapping Kernels for Trees is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with Mapping Kernels for Trees.  If not, see <http://www.gnu.org/licenses/>.

  For more details about this code, please look at the companion paper
  "Mapping Kernels for Trees", K. Shin, M.Cuturi, T.Kuboyama, Proceedings of
  the International Conference on Machine Learning 2011.

 */

case class FingerPrint(height: Int, length: Int) {
  def <(that: FingerPrint): Boolean = {
    height < that.height ||
    height == that.height && length < that.length
  }
  override def toString: String = {
    "["+height+" , "+length+"]"
  }
}


class RefNode(val entity: Vertex) {
  var prev: RefNode = null
  var next: List[RefNode] = Nil
  var signature: Signature = new Signature(List())
  var referencedFrom: Set[Int] = Set()
  var size: Int = 0
  def cut: RefNode = prev
  def take: RefNode = entity.refAsTree
  def branch: RefNode = entity.refOfChildren
  var cut_and_branch: RefNode = null
  def label: String = entity.label
  def height: Int = entity.height
  def depth: Int = entity.depth
  def children: List[RefNode] = entity.children.map(_.refAsTree).toList

  def fingerprint: FingerPrint = {
    FingerPrint(signature.height, signature.length)
  }

  def <(that: RefNode): Boolean = {
    signature.height < that.signature.height ||
    signature.height == that.signature.height &&
    signature.length < that.signature.length
  }

  def appendToSignature(i: Int) {
    signature = signature.append(i)
  }


  def show(i: Int): String = {
    var str: String = ""
    for(j <- 1 to i) str += "  "
    entity.refAsTree match{
      case null => str += "null"
      case _ => str += entity.refAsTree
    }
    str += ": Label: " + entity.label
    str += ": Height : "+entity.height
    str += ": Children :"
    entity.refOfChildren match{
      case null => str += "null"
      case _ => str += entity.refOfChildren
    }
    str += "\r\n"
    str
  }


  def showChain(i: Int): String = {
    var str: String = this.show(i)
    next.foreach(x => str += x.show(i+1))
    str
  }

  def showTree(i: Int): String = {
    var str: String = this.show(i)
    entity.children.map(_.refAsTree).foreach(x => str += x.showTree(i+1))
    str
  }

  def show: String = {
    val l = this.children.reverse
    var s: String = ""
    if(l isEmpty)
      s = this.label
    else
      s = l.init.foldLeft(this.label+"(")(_+_.show+" ")+l.last.show+")"
    if(this.prev != null)
      s += "|" + this.prev.show
    s
  }
}
